module intervalmath
    implicit none
    private

    public :: realinterval, intinterval
    type realinterval
        real :: lower, upper
    end type realinterval

    type intinterval
        integer :: lower, upper
    end type intinterval

    public :: addintervals, subtractintervals, multintervals
    public :: printinterval
    interface addintervals
        module procedure addintintervals, addrealintervals
    end interface addintervals
    interface subtractintervals
        module procedure subtractintintervals, subtractrealintervals
    end interface subtractintervals
    interface multintervals
        module procedure multintintervals, multrealintervals
    end interface multintervals
    interface printinterval
        module procedure printintinterval, printrealinterval
    end interface printinterval

contains

function addintintervals(a, b) result(c)
    implicit none
    type(intinterval), intent(in) :: a, b
    type(intinterval) :: c

    c % lower = a % lower + b % lower
    c % upper = a % upper + b % upper
end function addintintervals

function subtractintintervals(a, b) result(c)
    implicit none
    type(intinterval), intent(in) :: a, b
    type(intinterval) :: c

    c % lower = a % lower - b % upper
    c % upper = a % upper - b % lower
end function subtractintintervals

function multintintervals(a, b) result(c)
    implicit none
    type(intinterval), intent(in) :: a, b
    type(intinterval) :: c

    real, dimension(4) :: prods

    prods = [a%lower*b%lower, a%lower*b%upper, &
             a%upper*b%lower, a%upper*b%upper]

    c % lower = minval(prods)
    c % upper = maxval(prods)
end function  multintintervals

subroutine printintinterval(a)
    implicit none
    type(intinterval), intent(in) :: a
    print '(A1,I5,A1,I5,A1)', '[',a%lower,',',a%upper,']'
end subroutine printintinterval

function addrealintervals(a, b) result(c)
    implicit none
    type(realinterval), intent(in) :: a, b
    type(realinterval) :: c

    c % lower = a % lower + b % lower
    c % upper = a % upper + b % upper
end function addrealintervals

function subtractrealintervals(a, b) result(c)
    implicit none
    type(realinterval), intent(in) :: a, b
    type(realinterval) :: c

    c % lower = a % lower - b % upper
    c % upper = a % upper - b % lower
end function subtractrealintervals

function multrealintervals(a, b) result(c)
    implicit none
    type(realinterval), intent(in) :: a, b
    type(realinterval) :: c

    real, dimension(4) :: prods

    prods = [a%lower*b%lower, a%lower*b%upper, &
             a%upper*b%lower, a%upper*b%upper]

    c % lower = minval(prods)
    c % upper = maxval(prods)
end function  multrealintervals

subroutine printrealinterval(a)
    implicit none
    type(realinterval), intent(in) :: a
    print '(A1,G9.3,A1,G9.3,A1)', '[',a%lower,',',a%upper,']'
end subroutine printrealinterval

end module intervalmath
