module intervalmath

    implicit none
    private
    type, public :: intinterval
        integer :: lower, upper
        contains 
            procedure :: length => intlength
    end type intinterval

    type, public :: realinterval
        real :: lower, upper
        contains 
            procedure :: length => reallength
    end type realinterval

    interface operator(+)
        module procedure addintintervals
        module procedure addrealintervals
    end interface operator(+)

    interface operator(-)
        module procedure subtractintintervals
        module procedure subtractrealintervals
    end interface operator(-)

    interface operator(*)
        module procedure multintintervals
        module procedure multrealintervals
    end interface operator(*)

    interface assignment(=)
        module procedure realIntervalFromArray
        module procedure intIntervalFromArray
    end interface assignment(=)

    interface printinterval
        module procedure printintinterval, printrealinterval
    end interface printinterval

    public :: operator(+),operator(-),operator(*), assignment(=)
    public :: printinterval
contains

function addintintervals(a, b) result(c)
    implicit none
    type(intinterval), intent(in) :: a, b
    type(intinterval) :: c

    c % lower = a % lower + b % lower
    c % upper = a % upper + b % upper
end function addintintervals

function subtractintintervals(a, b) result(c)
    implicit none
    type(intinterval), intent(in) :: a, b
    type(intinterval) :: c

    c % lower = a % lower - b % upper
    c % upper = a % upper - b % lower
end function subtractintintervals

function multintintervals(a, b) result(c)
    implicit none
    type(intinterval), intent(in) :: a, b
    type(intinterval) :: c

    integer, dimension(4) :: prods

    prods = [a%lower*b%lower, a%lower*b%upper, &
             a%upper*b%lower, a%upper*b%upper]

    c % lower = minval(prods)
    c % upper = maxval(prods)
end function  multintintervals

subroutine printintinterval(a)
    implicit none
    type(intinterval), intent(in) :: a
    print '(A1,I5,A1,I5,A1)', '[',a%lower,',',a%upper,']'
end subroutine printintinterval

function addrealintervals(a, b) result(c)
    implicit none
    type(realinterval), intent(in) :: a, b
    type(realinterval) :: c

    c % lower = a % lower + b % lower
    c % upper = a % upper + b % upper
end function addrealintervals

function subtractrealintervals(a, b) result(c)
    implicit none
    type(realinterval), intent(in) :: a, b
    type(realinterval) :: c

    c % lower = a % lower - b % upper
    c % upper = a % upper - b % lower
end function subtractrealintervals

function multrealintervals(a, b) result(c)
    implicit none
    type(realinterval), intent(in) :: a, b
    type(realinterval) :: c

    real, dimension(4) :: prods

    prods = [a%lower*b%lower, a%lower*b%upper, &
             a%upper*b%lower, a%upper*b%upper]

    c % lower = minval(prods)
    c % upper = maxval(prods)
end function  multrealintervals

subroutine printrealinterval(a)
    implicit none
    type(realinterval), intent(in) :: a
    print '(A1,G9.3,A1,G9.3,A1)', '[',a%lower,',',a%upper,']'
end subroutine printrealinterval

subroutine realIntervalFromArray(ri,a)
    implicit none
    real, dimension(2), intent(in) :: a
    type(realinterval), intent(out) :: ri
    ri % lower = minval(a)
    ri % upper = maxval(a)
end subroutine realIntervalFromArray

subroutine intIntervalFromArray(ii,a)
    implicit none
    integer, dimension(2), intent(in) :: a
    type(intinterval), intent(out) :: ii
    ii % lower = minval(a)
    ii % upper = maxval(a)
end subroutine intIntervalFromArray

function intlength(ii) 
    implicit none
    class(intinterval), intent(in) :: ii
    integer :: intlength

    intlength = (ii%upper - ii % lower)
end function intlength

function reallength(ri) 
    implicit none
    class(realinterval), intent(in) :: ri
    real:: reallength

    reallength = (ri%upper - ri%lower)
end function reallength

end module intervalmath
