!
! F77-ized version of a Fortran90 TVD split MHD code
!
!  Original F90 code  copyright (C) 2001,2003, Ue-Li Pen
!  written November 2001 by Ue-Li Pen, pen@cita.utoronto.ca
!
! see http://arxiv.org/abs/astro-ph/0305088
! or http://www.cita.utoronto.ca/~pen/MHD
!
! This code is licencensed under the GPL
!
program hydro
  use pbm, only : outputpbm
  use solver, only : nvars, timestep, idens, initialconditions

  implicit none
  integer :: n,nx,ny
  integer, parameter :: defaultn = 100
  real  :: t,dt
  integer :: iter
  real, allocatable, dimension(:,:,:) :: u[:]
  integer :: start  ! our starting x in the global array
  character(len=30) :: arg
  character(len=3) :: imgstr
  integer :: thisimg, numimg
  real :: mindens[*], maxdens[*]

  ! get domain size from command line; if nothing
  ! on command line, prompt user

  if (command_argument_count() < 1) then
      n = defaultn
  else
      call get_command_argument(1, arg)
      read( arg,'(I30)'), n
      if (n < 2 .or. n > 500) then
         print *,'invalid n = ', trim(arg), ' using 100'
         n = 100
      endif
  endif

  nx = n
  ny = n
  allocate(u(nvars,nx+2,ny)[*])

  ! which image are we?
  thisimg = this_image()
  numimg  = num_images()
  start = (-(numimg+1)/2 + thisimg) * n 
  write(imgstr,'(I3)') thisimg

  call initialconditions(u,start)
  call getminmax(u(idens,:,:),mindens,maxdens)
  call outputpbm(u,imgstr//'-ics.pbm',idens,minv=mindens, maxv=maxdens)
  t=0
  timesteps: do iter=1,175
      call timestep(u,dt)
      t = t + 2.*dt
      if ((mod(iter,10).eq.1) .and. (thisimg == 1)) print *, iter, 'dt = ', dt, ' t = ', t
  end do timesteps

  sync all()
  call getminmax(u(idens,:,:),mindens,maxdens)
  call outputpbm(u,imgstr//'-final-dens.pbm',idens,minv=mindens, maxv=maxdens)
  deallocate(u)

contains
  subroutine getminmax(u,mindens,maxdens)
     real, dimension(:,:), intent(IN) :: u
     real, intent(OUT) :: mindens,maxdens
     
     real, save, codimension[*] :: locmin, locmax
     integer :: i, num
 
     num = num_images()
     locmin = minval(u)
     locmax = maxval(u)
     mindens = 1.e+19
     maxdens =-1.e+19
     sync all()
     do i=1,num
        mindens=min(mindens,locmin[i])
        maxdens=max(maxdens,locmax[i])
     enddo
  end subroutine getminmax

end program hydro
