module pbm
  implicit none

  private :: color
  public :: outputpbm

contains

pure function color(val, lo, hi) result(rgb)
  integer, dimension(3) :: rgb
  real, intent(IN) :: val, lo, hi

  real :: scaleval 

  scaleval = (val-lo)/(hi-lo) * 255.
  rgb(1) = scaleval*3
  if (rgb(1) > 255) rgb(1) = 255
  rgb(2) = scaleval
  rgb(3) = 0
end function color

subroutine outputpbm(var,filename,varnum,minv,maxv)
  character(len=*), intent(IN), optional :: filename
  integer, intent(IN), optional :: varnum
  real, dimension(:,:,:), intent(IN) :: var
  integer, dimension(:,:,:), allocatable :: rgb
  real, optional, intent(IN) :: minv, maxv
  real :: lo, hi
  integer :: i,j,c
  integer :: nx, ny, nvars
  character(len=30) :: chosenfilename
  integer :: chosenvarnum

  if (present(varnum)) then
      chosenvarnum = varnum
  else
      chosenvarnum = 1
  endif
  if (present(filename)) then
      chosenfilename = filename
  else
      chosenfilename = 'output.pbm'
  endif

  nvars = size(var,1)
  nx = size(var,2)
  ny = size(var,3)

  if (chosenvarnum > nvars) then
      print *, 'Invalid variable number ', chosenvarnum
  else
      allocate(rgb(3,nx,ny))

      if (present(minv)) then
        lo = minv
      else
        lo = minval(var(chosenvarnum,:,:))
      endif
      if (present(maxv)) then
        hi = maxv
      else
        hi = maxval(var(chosenvarnum,:,:))
      endif

      forall (i=1:nx,j=1:ny)
          rgb(1:3,i,j) = color(var(chosenvarnum,i,j), lo, hi)
      end forall
  
      open(unit=44,file=chosenfilename)
      write(44,'(A)') 'P3'
      write(44,'(2(1x,i4))') nx, ny
      write(44,'(i3)'), 255
      do j=ny,1,-1
         write(44,'(12(i4,1x))') ((rgb(c,i,j),c=1,3), i=1,nx)
      enddo
     
      deallocate(rgb)
      close(44)
  endif
end subroutine outputpbm

end module pbm
