module pbm
  implicit none

  private :: color
  public :: outputpbm

contains

pure subroutine color(val, lo, hi, rgb)
  real, intent(IN) :: val, lo, hi
  integer,dimension(3), intent(OUT) :: rgb

  real :: scaleval 

  scaleval = (val-lo)/(hi-lo) * 255.
  rgb(1) = scaleval*3
  if (rgb(1) > 255) rgb(1) = 255
  rgb(2) = scaleval
  rgb(3) = 0
end subroutine color

subroutine outputpbm(var,nvars,nx,ny,filename,varnum)
  integer, intent(IN) :: nvars
  integer, intent(IN) :: nx, ny
  real, dimension(nvars,nx,ny), intent(IN) :: var
  character(len=*), intent(IN), optional :: filename
  integer, intent(IN),optional :: varnum

  integer, dimension(nvars,nx,ny) :: rgb
  real :: lo, hi
  integer :: i,j,c
  integer :: chosenvar
  character(len=30) :: chosenfilename

  if (present(filename)) then
      chosenfilename=filename
  else
      chosenfilename='output.pbm'
  endif

  if (present(varnum)) then
      chosenvar = varnum
  else
      chosenvar = 1
  endif

  lo = 1.e+9
  hi = -1.e+9

  do i=1,nx
    do j=1,ny
       lo = min(lo,var(chosenvar,i,j)) 
       hi = max(hi,var(chosenvar,i,j)) 
    enddo
  enddo

  do i=1,nx
    do j=1,ny
       call color(var(chosenvar,i,j), lo, hi, rgb(1,i,j))
    enddo
  enddo

  open(unit=44,file=chosenfilename)
  write(44,'(A)') 'P3'
  write(44,'(2(1x,i4))') nx, ny
  write(44,'(i3)'), 255
  do j=ny,1,-1
     write(44,'(12(i4,1x))') ((rgb(c,i,j),c=1,3), i=1,nx)
  enddo
  close(44)
end subroutine outputpbm

end module pbm
